/*
 *  (Gnono) Card game for GNOME
 *
 *  Copyright (C) 2001-2002 Dennis J Houy <djhouy@paw.co.za>
 *            (C) 2006      Yaakov Selkowitz <yselkowitz@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *  $Id: ai.c,v 1.4 2007/01/04 06:06:30 yselkowitz Exp $
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/garray.h>
#include <glib/gmessages.h>
#include <glib/gtypes.h>

#include "cards.h"
#include "game.h"
#include "interface.h"
#include "players.h"
#include "ai.h"

gboolean computer_play_wild_4 (void);
gboolean computer_play_take_2 (void);
gboolean computer_play_skip (void);
gboolean computer_play_reverse (void);
gboolean computer_play_color (void);
gboolean computer_play_type (void);
gboolean computer_play_wild (void);
void computer_pick_up (void);
gint computer_highest_color (gint card_index);

void
computer_play (gboolean pickup)
{
    if (!game_in_progress ()) return;

    if (!computer_play_wild_4 ()) {
        if (!computer_play_take_2 ()) {
            if (!computer_play_skip ()) {
                if (!computer_play_reverse ()) {
                    if (!computer_play_color ()) {
                        if (!computer_play_type ()) {
                            if (!computer_play_wild ()) {
                                if (pickup) {
                                    computer_pick_up ();
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}

gboolean
computer_play_wild_4 ()
{
    gint c;
    Player *player;
    Card *card;

    player = get_player (get_current_player ());

    for (c = 0; c < player->cards->len; c++) {
        card = (Card*) g_ptr_array_index (player->cards, c);

        if ((card->type == GN_CARD_WILD_DRAW_4) && valid_play (get_current_player (), c)) {
            card->color = computer_highest_color (c);

#ifdef DEBUG
            g_message ("%s - %d", "Playing color", card->color);
#endif

            play_card (get_current_player (), c);

            return TRUE;
        }
    }

    return FALSE;
}

gboolean
computer_play_take_2 ()
{
    gint c;
    Player *player;
    Card *card;

    player = get_player (get_current_player ());

    for (c = 0; c < player->cards->len; c++) {
        card = (Card*) g_ptr_array_index (player->cards, c);

        if ((card->type == GN_CARD_TAKE_2) && valid_play (get_current_player (), c)) {
            play_card (get_current_player (), c);

            return TRUE;
        }
    }

    return FALSE;
}

gboolean
computer_play_skip ()
{
    gint c;
    Player *player;
    Card *card;

    player = get_player (get_current_player ());

    for (c = 0; c < player->cards->len; c++) {
        card = (Card*) g_ptr_array_index (player->cards, c);

        if ((card->type == GN_CARD_SKIP) && valid_play (get_current_player (), c)) {
            play_card (get_current_player (), c);

            return TRUE;
        }
    }

    return FALSE;
}

gboolean
computer_play_reverse ()
{
    gint c;
    Player *player;
    Card *card;

    player = get_player (get_current_player ());

    for (c = 0; c < player->cards->len; c++) {
        card = (Card*) g_ptr_array_index (player->cards, c);

        if ((card->type == GN_CARD_REVERSE) && valid_play (get_current_player (), c)) {
            play_card (get_current_player (), c);

            return TRUE;
        }
    }

    return FALSE;
}

gboolean
computer_play_color ()
{
    gint c;
    Player *player;
    Card *card;

    player = get_player (get_current_player ());

    for (c = 0; c < player->cards->len; c++) {
        card = (Card*) g_ptr_array_index (player->cards, c);

        if ((card->type != GN_CARD_WILD) && (card->type != GN_CARD_WILD_DRAW_4)) {
            if ((card->color == discard_top_card ()->color) && valid_play (get_current_player (), c)) {
                play_card (get_current_player (), c);
                return TRUE;
            }
        }
    }

    return FALSE;
}

gboolean
computer_play_type ()
{
    gint c;
    Player *player;
    Card *card;

    player = get_player (get_current_player ());

    for (c = 0; c < player->cards->len; c++) {
        card = (Card*) g_ptr_array_index (player->cards, c);

        if ((card->type != GN_CARD_WILD) && (card->type != GN_CARD_WILD_DRAW_4)) {
            if ((card->type == discard_top_card ()->type) && valid_play (get_current_player (), c)) {
                play_card (get_current_player (), c);
                return TRUE;
            }
        }
    }

    return FALSE;
}

gboolean
computer_play_wild ()
{
    gint c;
    Player *player;
    Card *card;

    player = get_player (get_current_player ());

    for (c = 0; c < player->cards->len; c++) {
        card = (Card*) g_ptr_array_index (player->cards, c);

        if (card->type == GN_CARD_WILD) {
            card->color = computer_highest_color (c);

#ifdef DEBUG
            g_message ("%s - %d", "Playing color", card->color);
#endif

            play_card (get_current_player (), c);

            return TRUE;
        }
    }

    return FALSE;
}

void
computer_pick_up ()
{
    player_cards_add (get_current_player (), deck_take_top ());
    computer_play (FALSE);
}

gint
computer_highest_color (gint card_index)
{
    gint c, cc = 0, rc = 0, gc = 0, bc = 0, yc = 0;
    CardColors _cc = GN_COLOR_NONE;
    Player *player;
    Card *card;
    gboolean have_wild_4 = FALSE;

    player = get_player (get_current_player ());

    for (c = 0; c < player->cards->len; c++) {
        if (c != card_index) {
            card = (Card*) g_ptr_array_index (player->cards, c);

            if (card->type == GN_CARD_WILD_DRAW_4)
                have_wild_4 = TRUE;

            switch (card->color) {
            case GN_COLOR_RED:
                rc++;
                break;
            case GN_COLOR_GREEN:
                gc++;
                break;
            case GN_COLOR_BLUE:
                bc++;
                break;
            case GN_COLOR_YELLOW:
                yc++;
                break;
            default:
                break;
            }
        }
    }

    if (have_wild_4) {
        if (rc == 0) {
            _cc = GN_COLOR_RED;
        } else if (gc == 0) {
            _cc = GN_COLOR_GREEN;
        } else if (bc == 0) {
            _cc = GN_COLOR_BLUE;
        } else if (yc == 0) {
            _cc = GN_COLOR_YELLOW;
        }
    } else {
        if (gc > rc) {
            cc = gc;
            _cc = GN_COLOR_GREEN;
        } else {
            cc = rc;
            _cc = GN_COLOR_RED;
        }

        if (bc > cc) {
            cc = bc;
            _cc = GN_COLOR_BLUE;
        }

        if (yc > cc)
            _cc = GN_COLOR_YELLOW;
    }

    if (_cc == GN_COLOR_NONE) {
        _cc = GN_COLOR_RED;
    }

    return _cc;
}
