/*
 *  (Gnono) Card game for GNOME
 *
 *  Copyright (C) 2001-2002 Dennis J Houy <djhouy@paw.co.za>
 *            (C) 2006      Yaakov Selkowitz <yselkowitz@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *  $Id: cards.c,v 1.4 2007/01/04 06:09:00 yselkowitz Exp $
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>			/* for rand() */
#include <glib/garray.h>
#include <glib/gmem.h>
#include <glib/gtypes.h>
#include <gdk-pixbuf/gdk-pixbuf-core.h>

#include "cards.h"
#include "game.h"
#include "interface.h"
#include "xpm/standard.h"

static GPtrArray *deck = NULL;
static GPtrArray *discard = NULL;

void deck_destroy (void);
void discard_destroy (void);
void turn_deck (void);

Card *
card_get_new (CardColors color, CardTypes type)
{
    Card *new_card = g_new (Card, 1);

    new_card->color = color;
    new_card->type = type;

    return new_card;
}

GdkPixbuf *
card_get_picture (CardColors color, CardTypes card)
{
    GdkPixbuf *p;

    p = gdk_pixbuf_new_from_xpm_data (card_get_xpm (color, card));

    return p;
}

const gchar **
card_get_xpm (CardColors color, CardTypes card)
{
  switch (card) {
  case GN_CARD_EMPTY:
    return empty_xpm;
  case GN_CARD_0:
    switch (color) {
    case GN_COLOR_RED: return r0_xpm;
    case GN_COLOR_GREEN: return g0_xpm;
    case GN_COLOR_BLUE: return b0_xpm;
    case GN_COLOR_YELLOW: return y0_xpm;
    default: break;
    }
  case GN_CARD_1:
    switch (color) {
    case GN_COLOR_RED: return r1_xpm;
    case GN_COLOR_GREEN: return g1_xpm;
    case GN_COLOR_BLUE: return b1_xpm;
    case GN_COLOR_YELLOW: return y1_xpm;
    default: break;
    }
  case GN_CARD_2:
    switch (color) {
    case GN_COLOR_RED: return r2_xpm;
    case GN_COLOR_GREEN: return g2_xpm;
    case GN_COLOR_BLUE: return b2_xpm;
    case GN_COLOR_YELLOW: return y2_xpm;
    default: break;
    }
  case GN_CARD_3:
    switch (color) {
    case GN_COLOR_RED: return r3_xpm;
    case GN_COLOR_GREEN: return g3_xpm;
    case GN_COLOR_BLUE: return b3_xpm;
    case GN_COLOR_YELLOW: return y3_xpm;
    default: break;
    }
  case GN_CARD_4:
    switch (color) {
    case GN_COLOR_RED: return r4_xpm;
    case GN_COLOR_GREEN: return g4_xpm;
    case GN_COLOR_BLUE: return b4_xpm;
    case GN_COLOR_YELLOW: return y4_xpm;
    default: break;
    }
  case GN_CARD_5:
    switch (color) {
    case GN_COLOR_RED: return r5_xpm;
    case GN_COLOR_GREEN: return g5_xpm;
    case GN_COLOR_BLUE: return b5_xpm;
    case GN_COLOR_YELLOW: return y5_xpm;
    default: break;
    }
  case GN_CARD_6:
    switch (color) {
    case GN_COLOR_RED: return r6_xpm;
    case GN_COLOR_GREEN: return g6_xpm;
    case GN_COLOR_BLUE: return b6_xpm;
    case GN_COLOR_YELLOW: return y6_xpm;
    default: break;
    }
  case GN_CARD_7:
    switch (color) {
    case GN_COLOR_RED: return r7_xpm;
    case GN_COLOR_GREEN: return g7_xpm;
    case GN_COLOR_BLUE: return b7_xpm;
    case GN_COLOR_YELLOW: return y7_xpm;
    default: break;
    }
  case GN_CARD_8:
    switch (color) {
    case GN_COLOR_RED: return r8_xpm;
    case GN_COLOR_GREEN: return g8_xpm;
    case GN_COLOR_BLUE: return b8_xpm;
    case GN_COLOR_YELLOW: return y8_xpm;
    default: break;
    }
  case GN_CARD_9:
    switch (color) {
    case GN_COLOR_RED: return r9_xpm;
    case GN_COLOR_GREEN: return g9_xpm;
    case GN_COLOR_BLUE: return b9_xpm;
    case GN_COLOR_YELLOW: return y9_xpm;
    default: break;
    }
  case GN_CARD_REVERSE:
    switch (color) {
    case GN_COLOR_RED: return rr_xpm;
    case GN_COLOR_GREEN: return gr_xpm;
    case GN_COLOR_BLUE: return br_xpm;
    case GN_COLOR_YELLOW: return yr_xpm;
    default: break;
    }
  case GN_CARD_SKIP:
    switch (color) {
    case GN_COLOR_RED: return rs_xpm;
    case GN_COLOR_GREEN: return gs_xpm;
    case GN_COLOR_BLUE: return bs_xpm;
    case GN_COLOR_YELLOW: return ys_xpm;
    default: break;
    }
  case GN_CARD_TAKE_2:
    switch (color) {
    case GN_COLOR_RED: return rp2_xpm;
    case GN_COLOR_GREEN: return gp2_xpm;
    case GN_COLOR_BLUE: return bp2_xpm;
    case GN_COLOR_YELLOW: return yp2_xpm;
    default: break;
    }
  case GN_CARD_WILD:
    return wild_xpm;
  case GN_CARD_WILD_DRAW_4:
    return wild4_xpm;
  case GN_CARD_BACK:
    return back_xpm;
  default:
    break;
  }

  return NULL;
}

void
deck_init (void)
{
    guint16 r, c, t;

    if (deck != NULL)
        deck_destroy ();

    deck = g_ptr_array_new ();

    for (c = 2; c <= (1 << 4); c *= 2) {
        deck_add_top (card_get_new (c, GN_CARD_0));
    }

    for (r = 0; r < 2; r++) {
        for (t = 2; t <= (1 << 12); t *= 2) {
            for (c = 2; c <= (1 << 4); c *= 2) {
                deck_add_top (card_get_new (c, t));
            }
        }
    }

    for (r = 0; r < 4; r++) {
        deck_add_top (card_get_new (GN_COLOR_NONE, GN_CARD_WILD));
        deck_add_top (card_get_new (GN_COLOR_NONE, GN_CARD_WILD_DRAW_4));
    }

    deck_shuffle ();
}

void
deck_destroy (void)
{
    if (deck == NULL)
        return;

    g_ptr_array_free (deck, TRUE);
}

void
deck_shuffle (void)
{
    guint16 i;

    for (i = 0; i < deck->len; i++) {
        deck_add_top (deck_take_index ((rand ()) % (i + 1)));
    }
}

void
deck_add_top (Card *card)
{
    g_ptr_array_add (deck, card);
}

gint
deck_length (void)
{
    return deck->len;
}

Card *
deck_take_index (guint16 index)
{
    return (Card*) g_ptr_array_remove_index (deck, index);
}

Card *
deck_take_top (void)
{
    Card *pick_card;

    if (deck->len == 0) turn_deck ();

    pick_card = (Card*) g_ptr_array_remove_index (deck, deck->len - 1);

    gci_pick_up_set_image ();

    return pick_card;
}

void
discard_init (void)
{
    if (discard != NULL)
        discard_destroy ();

    discard = g_ptr_array_new ();
    discard_add_top (deck_take_top ());

    while ((discard_top_card ()->type == GN_CARD_WILD) ||
            (discard_top_card ()->type == GN_CARD_WILD_DRAW_4)) {
        discard_add_top (deck_take_top ());
    }

    gci_discard_show_color ();
}

void
discard_destroy (void)
{
    g_ptr_array_free (discard, TRUE);
}

void
discard_add_top (Card *card)
{
    g_ptr_array_add (discard, card);
    gci_discard_set_image ();
}

Card *
discard_top_card (void)
{
    return (Card*) g_ptr_array_index (discard, discard->len - 1);
}

void
turn_deck (void)
{
    Card *top_card;

    top_card = (Card*) g_ptr_array_remove_index (discard, discard->len - 1);

    while (discard->len > 0) {
        g_ptr_array_add (deck, (Card*) g_ptr_array_remove_index (discard, discard->len - 1));
    }

    discard_add_top (top_card);
}
