/*
 *  (Gnono) Card game for GNOME
 *
 *  Copyright (C) 2001-2002 Dennis J Houy <djhouy@paw.co.za>
 *            (C) 2006      Yaakov Selkowitz <yselkowitz@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *  $Id: cards.h,v 1.4 2007/01/04 06:09:00 yselkowitz Exp $
 */
#ifndef __GN_CARDS_H__
#define __GN_CARDS_H__

#include <glib/gtypes.h>
#include <gdk-pixbuf/gdk-pixbuf-core.h>

static const gdouble GN_CARD_HEIGHT = 98.0;
static const gdouble GN_CARD_WIDTH = 68.0;

typedef enum {
    GN_COLOR_NONE   = 1 << 0,
    GN_COLOR_RED    = 1 << 1,
    GN_COLOR_GREEN  = 1 << 2,
    GN_COLOR_BLUE   = 1 << 3,
    GN_COLOR_YELLOW = 1 << 4
} CardColors;

typedef enum {
    GN_CARD_EMPTY       = -1,
    GN_CARD_0           = 1 << 0,
    GN_CARD_1           = 1 << 1,
    GN_CARD_2           = 1 << 2,
    GN_CARD_3           = 1 << 3,
    GN_CARD_4           = 1 << 4,
    GN_CARD_5           = 1 << 5,
    GN_CARD_6           = 1 << 6,
    GN_CARD_7           = 1 << 7,
    GN_CARD_8           = 1 << 8,
    GN_CARD_9           = 1 << 9,
    GN_CARD_REVERSE     = 1 << 10,
    GN_CARD_SKIP        = 1 << 11,
    GN_CARD_TAKE_2      = 1 << 12,
    GN_CARD_WILD        = 1 << 13,
    GN_CARD_WILD_DRAW_4 = 1 << 14,
    GN_CARD_BACK        = 1 << 15
} CardTypes;

typedef struct {
    CardColors color;
    CardTypes type;
} Card;

Card *card_get_new (CardColors color, CardTypes type);
GdkPixbuf *card_get_picture (CardColors color, CardTypes card);
const gchar ** card_get_xpm (CardColors color, CardTypes card);

void deck_init (void);
void deck_destroy (void);
void deck_shuffle (void);
void deck_add_top (Card *card);
gint deck_length (void);
Card *deck_take_index (guint16 index);
Card *deck_take_top (void);

void discard_init (void);
void discard_destroy (void);
void discard_add_top (Card *card);
Card *discard_top_card (void);

#endif /* __GN_CARDS_H__ */
