/*
 *  (Gnono) Card game for GNOME
 *
 *  Copyright (C) 2001-2002 Dennis J Houy <djhouy@paw.co.za>
 *            (C) 2006      Yaakov Selkowitz <yselkowitz@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *  $Id: preferences.c,v 1.4 2007/01/04 06:09:01 yselkowitz Exp $
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <libgnome/gnome-config.h>
#include <libgnomeui/gnome-dialog.h>
#include <libgnomeui/gnome-messagebox.h>
#include <libgnomeui/gnome-propertybox.h>

#include "cards.h"
#include "game.h"
#include "interface.h"
#include "preferences.h"

GtkWidget *_dialog = NULL;

GtkWidget *_radio_cards_7, *_radio_cards_11;
GtkWidget *_radio_score_100, *_radio_score_250, *_radio_score_500;
GtkWidget *_entry_name_human, *_entry_name_computer;

gboolean _loaded = FALSE;

gint apply_cb (GnomePropertyBox *propertybox, gint page, gpointer user_data);
gint close_cb (GnomePropertyBox *propertybox, gint page, gpointer user_data);

void add_game_play_page (void);
void add_names_page (void);

void load_game_play_prefs (void);
void load_names_prefs (void);

void save_game_play_prefs (void);
void save_names_prefs (void);

void prefs_changed_cb (void);

void
dlg_preferences (GtkWidget *caller, gpointer data)
{
    _dialog = gnome_property_box_new ();

    gtk_window_set_title (GTK_WINDOW (_dialog), _("Gnono: Preferences"));
    gnome_dialog_set_parent (GNOME_DIALOG (_dialog), main_app_window ());
    gtk_window_set_modal (GTK_WINDOW (_dialog), TRUE);

    add_game_play_page ();
    add_names_page ();

    g_signal_connect (GTK_OBJECT (_dialog), "close",
                        GTK_SIGNAL_FUNC (close_cb), NULL);

    g_signal_connect (GTK_OBJECT (_dialog), "apply",
                        GTK_SIGNAL_FUNC (apply_cb), NULL);

    _loaded = TRUE;

    gnome_dialog_run (GNOME_DIALOG (_dialog));
}

void
add_game_play_page (void)
{
    GtkWidget *hbox, *vbox, *frame, *label, *separator;
    GSList *radio_group = NULL;

    /* Create a label for the new tab */
    label = gtk_label_new (_("Game Play"));

    hbox = gtk_hbox_new (FALSE, 0);

    /* Create the cards to deal frame */
    frame = gtk_frame_new (_("Cards to Deal"));
    gtk_container_set_border_width (GTK_CONTAINER (frame), 2);
    gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 0);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 2);
    gtk_container_add (GTK_CONTAINER (frame), vbox);

    _radio_cards_7 = gtk_radio_button_new_with_label (radio_group, _("7"));
    gtk_container_set_border_width (GTK_CONTAINER (_radio_cards_7), 2);
    radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (_radio_cards_7));
    gtk_box_pack_start (GTK_BOX (vbox), _radio_cards_7, FALSE, FALSE, 0);

    g_signal_connect (GTK_OBJECT (_radio_cards_7), "toggled",
                        GTK_SIGNAL_FUNC (prefs_changed_cb), NULL);

    _radio_cards_11 = gtk_radio_button_new_with_label (radio_group, _("11"));
    gtk_container_set_border_width (GTK_CONTAINER (_radio_cards_11), 2);
    gtk_box_pack_start (GTK_BOX (vbox), _radio_cards_11, FALSE, FALSE, 0);

    g_signal_connect (GTK_OBJECT (_radio_cards_11), "toggled",
                        GTK_SIGNAL_FUNC (prefs_changed_cb), NULL);

    /* Add a separator between the two frames */
    separator = gtk_vseparator_new ();
    gtk_box_pack_start (GTK_BOX (hbox), separator, FALSE, FALSE, 0);

    /* Create the playout score frame */
    frame = gtk_frame_new (_("Playout Score"));
    gtk_container_set_border_width (GTK_CONTAINER (frame), 2);
    gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 0);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox), 2);
    gtk_container_add (GTK_CONTAINER (frame), vbox);

    radio_group = NULL;

    _radio_score_100 = gtk_radio_button_new_with_label (radio_group, _("100"));
    gtk_container_set_border_width (GTK_CONTAINER (_radio_score_100), 2);
    radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (_radio_score_100));
    gtk_box_pack_start (GTK_BOX (vbox), _radio_score_100, FALSE, FALSE, 0);

    g_signal_connect (GTK_OBJECT (_radio_score_100), "toggled",
                        GTK_SIGNAL_FUNC (prefs_changed_cb), NULL);

    _radio_score_250 = gtk_radio_button_new_with_label (radio_group, _("250"));
    gtk_container_set_border_width (GTK_CONTAINER (_radio_score_250), 2);
    radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (_radio_score_100));
    gtk_box_pack_start (GTK_BOX (vbox), _radio_score_250, FALSE, FALSE, 0);

    g_signal_connect (GTK_OBJECT (_radio_score_250), "toggled",
                        GTK_SIGNAL_FUNC (prefs_changed_cb), NULL);

    _radio_score_500 = gtk_radio_button_new_with_label (radio_group, _("500"));
    gtk_container_set_border_width (GTK_CONTAINER (_radio_score_500), 2);
    gtk_box_pack_start (GTK_BOX (vbox), _radio_score_500, FALSE, FALSE, 0);

    g_signal_connect (GTK_OBJECT (_radio_score_500), "toggled",
                        GTK_SIGNAL_FUNC (prefs_changed_cb), NULL);

    /* Show everything and append the new page to the dialog */
    gtk_widget_show_all (hbox);

    gnome_property_box_append_page (GNOME_PROPERTY_BOX (_dialog), hbox, label);

    load_game_play_prefs ();
}

void
add_names_page (void)
{
    GtkWidget *table, *label, *ilabel;

    /* Create a label for the new tab */
    label = gtk_label_new (_("Names"));

    table = gtk_table_new (2, 2, FALSE);
    gtk_container_set_border_width (GTK_CONTAINER (table), 4);

    ilabel = gtk_label_new (_("Human Player: "));
    gtk_misc_set_alignment (GTK_MISC (ilabel), 0.0, 0.5);

    gtk_table_attach (GTK_TABLE (table),
                      GTK_WIDGET (ilabel),
                      0, 1, 0, 1,
                      GTK_FILL,
                      0,
                      2, 2);

    ilabel = gtk_label_new (_("Computer Player: "));
    gtk_misc_set_alignment (GTK_MISC (ilabel), 0.0, 0.5);
    gtk_table_attach (GTK_TABLE (table),
                      GTK_WIDGET (ilabel),
                      0, 1, 1, 2,
                      GTK_FILL,
                      0,
                      2, 2);

    _entry_name_human = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table),
                      GTK_WIDGET (_entry_name_human),
                      1, 2, 0, 1,
                      GTK_EXPAND + GTK_FILL,
                      0,
                      2, 2);

    g_signal_connect (GTK_OBJECT (_entry_name_human), "changed",
                        GTK_SIGNAL_FUNC (prefs_changed_cb), NULL);

    _entry_name_computer = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table),
                      GTK_WIDGET (_entry_name_computer),
                      1, 2, 1, 2,
                      GTK_EXPAND + GTK_FILL,
                      0,
                      2, 2);

    g_signal_connect (GTK_OBJECT (_entry_name_computer), "changed",
                        GTK_SIGNAL_FUNC (prefs_changed_cb), NULL);

    /* Show everything and append the new page to the dialog */
    gtk_widget_show_all (table);

    gnome_property_box_append_page (GNOME_PROPERTY_BOX (_dialog), table, label);

    load_names_prefs ();
}

void
load_game_play_prefs (void)
{
    gint cards_to_deal, playout_score;

    cards_to_deal = gnome_config_private_get_int ("Gnono/GamePlay/CardsToDeal=7");

    if (cards_to_deal == 11) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_radio_cards_11), TRUE);
    } else {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_radio_cards_7), TRUE);
    }

    playout_score = gnome_config_private_get_int ("Gnono/GamePlay/PlayoutScore=100");

    if (playout_score == 500) {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_radio_score_500), TRUE);
    } else if (playout_score == 250){
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_radio_score_250), TRUE);
    } else {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_radio_score_100), TRUE);
    }
}

void
load_names_prefs (void)
{
    gchar *value, *key;

    key = g_strdup_printf ("Gnono/Names/%s=%s", g_get_user_name (), g_get_user_name ());

    value = gnome_config_private_get_string (key);

    gtk_entry_set_text (GTK_ENTRY (_entry_name_human), value);

    g_free (key);
    g_free (value);

    key = g_strdup_printf ("Gnono/Names/AI=%s", _("Computer"));

    value = gnome_config_private_get_string (key);

    gtk_entry_set_text (GTK_ENTRY (_entry_name_computer), value);

    g_free (key);
    g_free (value);
}

void
save_game_play_prefs (void)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (_radio_cards_7))) {
        gnome_config_private_set_int ("Gnono/GamePlay/CardsToDeal", 7);
    } else {
        gnome_config_private_set_int ("Gnono/GamePlay/CardsToDeal", 11);
    }

    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (_radio_score_100))) {
        gnome_config_private_set_int ("Gnono/GamePlay/PlayoutScore", 100);
    } else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (_radio_score_250))) {
        gnome_config_private_set_int ("Gnono/GamePlay/PlayoutScore", 250);
    } else {
        gnome_config_private_set_int ("Gnono/GamePlay/PlayoutScore", 500);
    }
}

void
save_names_prefs (void)
{
    gchar *value, *key;

    value = gtk_editable_get_chars (GTK_EDITABLE (_entry_name_human), 0, -1);

    key = g_strdup_printf ("Gnono/Names/%s", g_get_user_name ());

    gnome_config_private_set_string (key, value);

    g_free (key);
    g_free (value);

    value = gtk_editable_get_chars (GTK_EDITABLE (_entry_name_computer), 0, -1);

    gnome_config_private_set_string ("Gnono/Names/AI", value);

    g_free (value);

    prompt_user_info ("Currently, changes to names will only take effect\nthe next time Gnono is loaded,\nor if the first game has not yet started.\nThis will be fixed in future versions.", GTK_STOCK_OK);
}

gint
apply_cb (GnomePropertyBox *propertybox, gint page, gpointer user_data)
{
    switch (page) {
    case -1:
        gnome_config_sync ();
        return TRUE;
    case 0:
        save_game_play_prefs ();
        break;
    case 1:
        save_names_prefs ();
    }

    return TRUE;
}

gint
close_cb (GnomePropertyBox *propertybox, gint page, gpointer caller)
{
    GtkWidget *dialog;

    gtk_widget_hide (GTK_WIDGET (_dialog));

    if (((gnome_config_private_get_int ("Gnono/GamePlay/CardsToDeal=7") != get_cards_to_deal ()) ||
        (gnome_config_private_get_int ("Gnono/GamePlay/PlayoutScore=100") != get_playout_score ())) &&
        game_in_progress ())
    {
        dialog = gnome_message_box_new (_("The changes you have made will not take effect until you start a new game.\nWould you like to start a new game now?"),
                                        GTK_STOCK_DIALOG_QUESTION,
                                        GTK_STOCK_YES,
                                        GTK_STOCK_NO,
                                        NULL);

        gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (main_app_window ()));
        gnome_dialog_set_default (GNOME_DIALOG (dialog), 0);
        gnome_dialog_set_close (GNOME_DIALOG (dialog), TRUE);
        gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

        if (gnome_dialog_run_and_close (GNOME_DIALOG (dialog)) == 0)
            game_new_cb (NULL, NULL);
    }

    return FALSE;
}

void
prefs_changed_cb (void)
{
    if (!_loaded) return;

    gnome_property_box_changed (GNOME_PROPERTY_BOX (_dialog));
}
